//V-USB based piezo drumpad by Felix Held
//This is the old and ugly version from 2008, but the new, clean implementation doesn't work at the moment.

//This is partly based on the V-USB-MIDI project (see http://cryptomys.de/horo/V-USB-MIDI/index.html)
//and Guido Naruhn's MIDI drumpad (see http://ispf.de/modules.php?name=News&file=article&sid=6)

/* Name: main.c
 * Project: AVR MIDI device on Low-Speed USB
 * Author: Martin Homuth-Rosemann
 * Creation Date: 2008-03-11
 * Copyright: (c) 2008 by Martin Homuth-Rosemann.
 * License: GPL v2 or later.
 *
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "usbdrv.h"


#define MidiCh 10 							// Midi Channel

#define ADCRef	(1<<REFS0)			//AVCC with external capacitor at AREF pin

// a lookup table for individual play and feel
#include "vtable.h"

unsigned char TData[6]={0,0,0,0,0,0};
unsigned char mdata[4]={0,0,0,0};

unsigned char Voice[6]=
{
	42, // Closed Hi Hat
	38, // Acoustic Snare
	45, // Low Tom
	47, // Low Mid Tom
	48, // Hi Mid Tom
	50//, // High Tom
	//49, //Crash Cymbal
	//53  // Ride Bell
};

unsigned char Peak[6]={0,0,0,0,0,0};
unsigned char LastPeak[6]={31,31,31,31,31,31};

unsigned char midiMsg[8];


#if DEBUG_LEVEL > 0
#	warning "Never compile production devices with debugging enabled"
#endif


// This deviceDescrMIDI[] is based on 
// http://www.usb.org/developers/devclass_docs/midi10.pdf
// Appendix B. Example: Simple MIDI Adapter (Informative)

// B.1 Device Descriptor
static PROGMEM char deviceDescrMIDI[] = {	/* USB device descriptor */
	18,			/* sizeof(usbDescriptorDevice): length of descriptor in bytes */
	USBDESCR_DEVICE,	/* descriptor type */
	0x10, 0x01,		/* USB version supported */
	0,			/* device class: defined at interface level */
	0,			/* subclass */
	0,			/* protocol */
	8,			/* max packet size */
	USB_CFG_VENDOR_ID,	/* 2 bytes */
	USB_CFG_DEVICE_ID,	/* 2 bytes */
	USB_CFG_DEVICE_VERSION,	/* 2 bytes */
	1,			/* manufacturer string index */
	2,			/* product string index */
	0,			/* serial number string index */
	1,			/* number of configurations */
};

// B.2 Configuration Descriptor
static PROGMEM char configDescrMIDI[] = {	/* USB configuration descriptor */
	9,			/* sizeof(usbDescrConfig): length of descriptor in bytes */
	USBDESCR_CONFIG,	/* descriptor type */
	66, 0,			/* total length of data returned (including inlined descriptors) */
	2,			/* number of interfaces in this configuration */
	1,			/* index of this configuration */
	0,			/* configuration name string index */
#if USB_CFG_IS_SELF_POWERED
	USBATTR_SELFPOWER,	/* attributes */
#else
	USBATTR_BUSPOWER,	/* attributes */
#endif
	USB_CFG_MAX_BUS_POWER / 2,	/* max USB current in 2mA units */

// B.3 AudioControl Interface Descriptors
// The AudioControl interface describes the device structure (audio function topology)
// and is used to manipulate the Audio Controls. This device has no audio function
// incorporated. However, the AudioControl interface is mandatory and therefore both
// the standard AC interface descriptor and the classspecific AC interface descriptor
// must be present. The class-specific AC interface descriptor only contains the header
// descriptor.

// B.3.1 Standard AC Interface Descriptor
// The AudioControl interface has no dedicated endpoints associated with it. It uses the
// default pipe (endpoint 0) for all communication purposes. Class-specific AudioControl
// Requests are sent using the default pipe. There is no Status Interrupt endpoint provided.
	/* descriptor follows inline: */
	9,			/* sizeof(usbDescrInterface): length of descriptor in bytes */
	USBDESCR_INTERFACE,	/* descriptor type */
	0,			/* index of this interface */
	0,			/* alternate setting for this interface */
	0,			/* endpoints excl 0: number of endpoint descriptors to follow */
	1,			/* */
	1,			/* */
	0,			/* */
	0,			/* string index for interface */

// B.3.2 Class-specific AC Interface Descriptor
// The Class-specific AC interface descriptor is always headed by a Header descriptor
// that contains general information about the AudioControl interface. It contains all
// the pointers needed to describe the Audio Interface Collection, associated with the
// described audio function. Only the Header descriptor is present in this device
// because it does not contain any audio functionality as such.
	/* descriptor follows inline: */
	9,			/* sizeof(usbDescrCDC_HeaderFn): length of descriptor in bytes */
	36,			/* descriptor type */
	1,			/* header functional descriptor */
	0x0, 0x01,		/* bcdADC */
	6, 0,			/* wTotalLength */
	1,			/* */
	1,			/* */

// B.4 MIDIStreaming Interface Descriptors

// B.4.1 Standard MS Interface Descriptor
	/* descriptor follows inline: */
	9,			/* length of descriptor in bytes */
	USBDESCR_INTERFACE,	/* descriptor type */
	1,			/* index of this interface */
	0,			/* alternate setting for this interface */
	1,			/* endpoints excl 0: number of endpoint descriptors to follow */
	1,			/* AUDIO */
	3,			/* MS */
	0,			/* unused */
	0,			/* string index for interface */

// B.4.2 Class-specific MS Interface Descriptor
	/* descriptor follows inline: */
	7,			/* length of descriptor in bytes */
	36,			/* descriptor type */
	1,			/* header functional descriptor */
	0x0, 0x01,		/* bcdADC */
	30, 0,			/* wTotalLength */

//B.4.4 MIDI OUT Jack Descriptor
	/* descriptor follows inline: */
	9,			/* length of descriptor in bytes */
	36,			/* descriptor type */
	3,			/* MIDI_OUT_JACK descriptor */
	1,			/* EMBEDDED bJackType */
	1,			/* bJackID */
	1,			/* No of input pins */
	2,			/* BaSourceID */
	1,			/* BaSourcePin */
	0,			/* iJack */


// B.5 Bulk OUT Endpoint Descriptors


//B.6 Bulk IN Endpoint Descriptors

//B.6.1 Standard Bulk IN Endpoint Descriptor
	/* descriptor follows inline: */
	9,			/* bLenght */
	USBDESCR_ENDPOINT,	/* bDescriptorType = endpoint */
	0x81,			/* bEndpointAddress IN endpoint number 1 */
	3,			/* bmAttributes: 2: Bulk, 3: Interrupt endpoint */
	8, 0,			/* wMaxPacketSize */
	10,			/* bIntervall in ms */
	0,			/* bRefresh */
	0,			/* bSyncAddress */

// B.6.2 Class-specific MS Bulk IN Endpoint Descriptor
	/* descriptor follows inline: */
	5,			/* bLength of descriptor in bytes */
	37,			/* bDescriptorType */
	1,			/* bDescriptorSubtype */
	1,			/* bNumEmbMIDIJack (0) */
	1,			/* baAssocJackID (0) */
};


uchar usbFunctionDescriptor(usbRequest_t * rq)
{
	if (rq->wValue.bytes[1] == USBDESCR_DEVICE)
	{
		usbMsgPtr = (uchar *) deviceDescrMIDI;
		return sizeof(deviceDescrMIDI);
	}
	else
	{		// must be config descriptor
		usbMsgPtr = (uchar *) configDescrMIDI;
		return sizeof(configDescrMIDI);
	}
}


//static uchar sendEmptyFrame;


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

uchar usbFunctionSetup(uchar data[8])
{
	usbRequest_t *rq = (void *) data;

	if ((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS)	// class request type
	{
		//  Prepare bulk-in endpoint to respond to early termination
		if ((rq->bmRequestType & USBRQ_DIR_MASK) == USBRQ_DIR_HOST_TO_DEVICE)
			;
	}

	return 0xff;
}


/*---------------------------------------------------------------------------*/
/* usbFunctionRead                                                           */
/*---------------------------------------------------------------------------*/

uchar usbFunctionRead(uchar * data, uchar len)
{
	data[0] = 0;
	data[1] = 0;
	data[2] = 0;
	data[3] = 0;
	data[4] = 0;
	data[5] = 0;
	data[6] = 0;

	return 7;
}



/*---------------------------------------------------------------------------*/
/* hardwareInit                                                              */
/*---------------------------------------------------------------------------*/

static void hardwareInit(void)
{
	uchar i, j;

	// activate pull-ups except on USB lines
	USB_CFG_IOPORT = (uchar) ~ ((1 << USB_CFG_DMINUS_BIT) | (1 << USB_CFG_DPLUS_BIT));
	// all pins input except USB (-> USB reset)
#ifdef USB_CFG_PULLUP_IOPORT	// use usbDeviceConnect()/usbDeviceDisconnect() if available
	USBDDR = 0;		// we do RESET by deactivating pullup
	usbDeviceDisconnect();
#else
	USBDDR = (1 << USB_CFG_DMINUS_BIT) | (1 << USB_CFG_DPLUS_BIT);
#endif

	j = 0;
	while (--j) {		// USB Reset by device only required on Watchdog Reset
		i = 0;
		while (--i);	// delay >10ms for USB reset
	}
#ifdef USB_CFG_PULLUP_IOPORT
	usbDeviceConnect();
#else
	USBDDR = 0;		//  remove USB reset condition
#endif

	// ADC Setup; PORTC is used for six potentiometer inputs.
	// no pullups on ADC port
	DDRC = 0;
	PORTC = 0;

	ADMUX = 0 | ADCRef | (1<<ADLAR);
	// enable, prescaler = 2^6 (-> 16Mhz / 64 = 250 kHz; bit too fast, but should work...)
	ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1);

	DDRB=0x3f;		// Port B als Ausgang
	PORTB=0x3f;
}

// return 10 bit analog value
unsigned char adc(uchar channel)
{
	ADMUX = channel | ADCRef | (1<<ADLAR);
	asm("nop");
	asm("nop");
	ADCSRA |= (1<<ADSC); // start single conversion
	while(ADCSRA & (1<<ADSC));
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	return ADCH;
}

void play(unsigned char Channel, unsigned char velocity)
{
	unsigned char delay;
	if(usbInterruptIsReady())
	{
		delay = 10;	//this ugly hack is needed to discharge the capacitor properly...
	}
	else
	{
		delay = 0;
		while(!usbInterruptIsReady());
	}
	
	midiMsg[0] = 0x09;		//note on
	midiMsg[1] = 0x90 + MidiCh;
	midiMsg[2] = Voice[Channel];
	midiMsg[3] = velocity;
	midiMsg[4] = 0x08;		//note off
	midiMsg[5] = 0x80 + MidiCh;
	midiMsg[6] = Voice[Channel];
	midiMsg[7] = 0x10;
	
	usbSetInterrupt(midiMsg, 8);
	
	while(delay)	//wait for the capacitor to discharge...
	{
		delay--;
	}
}


int main(void)
{
	unsigned char inLevel=0;
	unsigned char Channel=0;

	hardwareInit();
	usbInit();

	sei();

	for (;;)
	{
		usbPoll();
		
		inLevel=adc(Channel);

		if(inLevel>3)
		{
			if(inLevel>Peak[Channel])
			{
				TData[Channel]=1;
				Peak[Channel]=inLevel;
			}
			else if(inLevel < Peak[Channel] && TData[Channel]==1) // Peak erreicht
			{
				PORTB&= ~(1<<Channel); // discharge capacicator
				TData[Channel]=0;
				play(Channel,pgm_read_byte(&velocity[Peak[Channel]]));
				PORTB|=(1<<Channel);
			}
		}
		else
		{
			Peak[Channel]=0;
		}
		Channel++;
		if(Channel==6) Channel=0;

	}
	return 0;
}

